@@ -8,7 +8,7 @@
 # 特記事項:
 # なし
 #
-# Copyright (c) 2012-2019, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+# Copyright (c) 2012-2020, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
 #
 #####################################################################
 package PGRex;
@@ -19,8 +19,10 @@ use Net::OpenSSH;
 use PGRex::command;
 use Class::Struct;
 use Fcntl;
+use Fcntl ':mode';
 use Errno;
 
+
 BEGIN {
     if ($ENV{'LANG'} =~ m/ja/i){
         eval qq{
@@ -46,7 +48,7 @@ struct Pg_dir_state => {
 };
 
 use constant {
-    VERSIONNUM       => "11.2",
+    VERSIONNUM       => "12.0",
     VERSIONINFO      => "[0] (pg-rex_operation_tools) [1]\n",
     CONFIG_PATH      => "/etc/",
     CONFIG_FILENAME  => "pg-rex_tools.conf",
@@ -57,7 +59,9 @@ use constant {
     RA_TMPDIR        => "/var/lib/pgsql/tmp",
     LOCK_FILENAME    => "PGSQL.lock",
     PID_FILENAME     => "pg-rex_tools.pid",
-    PID_FILEDIR      => "/var/run"
+    PID_FILEDIR      => "/var/run",
+    STANDBY_SIGNAL   => "standby.signal",
+    RECOVERY_SIGNAL  => "recovery.signal"
 };
 
 our $additional_information_mode = 0;
@@ -92,7 +96,7 @@ sub pacemaker_online{
     my @crm_results;
     my $exit_code;
 
-    $results[0] = `$CRM_MON -fA -1 2>&1`;
+    $results[0] = `$PCS status --full 2>&1`;
     $results[1] = $? >> 8;
 
     $exit_code = $results[1];
@@ -101,13 +105,13 @@ sub pacemaker_online{
         printlog("ERROR", COMMON_MS0001);
     }
 
-    # crm_mon の結果を行ごとの配列に格納する
+    # pcs status --full の結果を行ごとの配列に格納する
     @crm_results = split (/\n/, $results[0]);
 
     # Pacemaker が Online だったら1を返却する
     foreach my $line (@crm_results){
-        # crm_mon の結果に「Online: [ ～ <マシンのホスト名>」の行が存在すると OK
-        if ($line =~ /Online\:\s+\[(\s+\S+)?\s+$my_node\s+/){
+        # pcs status --full の結果に「Online: [ ～ <マシンのホスト名>」の行が存在すると OK
+        if ($line =~ /Online\:\s+\[(\s+\S+)?\s.*$my_node\s+/){
             return 1;
         }
     }
@@ -122,7 +126,7 @@ sub pgrex_failed_action{
     my @crm_results;
     my $exit_code;
 
-    $results[0] = `$CRM_MON -fA -1 2>&1`;
+    $results[0] = `$PCS status --full 2>&1`;
     $results[1] = $? >> 8;
 
     $exit_code = $results[1];
@@ -131,19 +135,19 @@ sub pgrex_failed_action{
         printlog("ERROR", COMMON_MS0001);
     }
 
-    # crm_mon の結果を行ごとの配列に格納する
+    # pcs status --full の結果を行ごとの配列に格納する
     @crm_results = split (/\n/, $results[0]);
 
     my $line_count = 0;
     my $line_tmp = 0;
-    # crm_mon の結果に failed Action の内容が存在していたら1を返却する
+    # pcs status --full の結果に failed Action の内容が存在していたら1を返却する
     foreach my $line (@crm_results){
         $line_count++;
 
-        # crm_mon の結果に「Failed actions:」の行が存在した後、
+        # pcs status --full の結果に「Failed Resource Actions:」の行が存在した後、
         # 1行後に「<PostgreSQLのPrimitiveのリソースID> ～ on <マシンのホスト名>」の行が存在する場合、
         # PG-REXリソースの故障が発生したと判断し、1 を返却する
-        if ($line =~ /Failed\s+actions:/){
+        if ($line =~ /Failed\s+Resource\s+Actions:/){
             $line_tmp = $line_count;
         }
         if ($line =~ /${pg_primitive_resource_id}_\S+_[0-9]+\s+on\s+$target_node/ && $line_count == $line_tmp + 1){
@@ -154,78 +158,16 @@ sub pgrex_failed_action{
 
 }
 
-
-sub network_running{
-    my ($my_node, $another_node, $iclan_ipaddr) = @_;
-    my $result;
-    my $attribute;
-    my $exit_code;
-    my $count;
-    my @matched;
-
-    # コマンド
-    # 「crm_attribute -t status -N <ノード名> --name ringnumber_N -G -q」※N=0,1
-    # の結果が2ノードとも"UP"であると OK
-    for ($count = 0; $count < 2; $count++){
-        # $iclan_ipaddr{'my_node'} に IP アドレスが1つしか格納されていない場合は、
-        # ringnumber_1 の属性値の確認は行わない
-        if (!defined(@{$$iclan_ipaddr{'my_node'}}[$count])){
-            last;
-        }
-        $result = `$CRM_ATTRIBUTE -t status -N $my_node --name ringnumber_$count -G -q 2> /dev/null`;
-        chomp $result;
-        
-        # crm_attribute の実行結果が "<IPアドレス> is UP"であれば、corosync.conf の設定と
-        # IPアドレスが一致しているかを確認する
-        # なお、crm_attribute の実行結果から取得できるIPアドレスのため、
-        # 厳密なIPv4の正規表現でIPアドレスを取得しない
-        if ($result =~ /^(\d+\.\d+\.\d+\.\d+)\sis\sUP/){
-            @matched = grep {$_ eq $1} @{$$iclan_ipaddr{'my_node'}};
-            if (@matched == 0){
-                return 0;
-            }
-        }
-        else {
-            return 0;
-        }
-    }
-    for ($count = 0; $count < 2; $count++){
-        # $iclan_ipaddr{'another_node'} に IP アドレスが1つしか格納されていない場合は、
-        # ringnumber_1 の属性値の確認は行わない
-        if (!defined(@{$$iclan_ipaddr{'another_node'}}[$count])){
-            last;
-        }
-        $result = `$CRM_ATTRIBUTE -t status -N $another_node --name ringnumber_$count -G -q 2> /dev/null`;
-        chomp $result;
-        
-        # crm_attribute の実行結果が "<IPアドレス> is UP"であれば、corosync.conf の設定と
-        # IPアドレスが一致しているかを確認する
-        # なお、crm_attribute の実行結果から取得できるIPアドレスのため、
-        # 厳密なIPv4の正規表現でIPアドレスを取得しない
-        if ($result =~ /^(\d+\.\d+\.\d+\.\d+)\sis\sUP/){
-            @matched = grep {$_ eq $1} @{$$iclan_ipaddr{'another_node'}};
-            if (@matched == 0){
-                return 0;
-            }
-        }
-        else {
-            return 0;
-        }
-    }
-    return 1;
-}
-
-
-sub master_running{
-    my ($my_node, $master_resource_id, $pg_primitive_resource_id, $ssh_info) = @_;
+sub primary_running{
+    my ($my_node, $primary_resource_id, $pg_primitive_resource_id, $ssh_info) = @_;
     my @results;
     my $command;
 
     ##
     # 以下の条件を満たしたとき結果を OK とする
     #   *  コマンド
-    #     「crm_resource -r <MasterリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
-    #      の結果が"resource <MasterリソースID> is running on: <マシンのホスト名> Master"である
+    #     「crm_resource -r <PrimaryリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
+    #      の結果が"resource <PrimaryリソースID> is running on: <マシンのホスト名> Primary"である
     #   *  コマンド
     #     「crm_attribute -l forever -N <マシンのホスト名> --name <PostgreSQLのPrimitiveのリソースID>-data-status -G -q」
     #      の結果が"LATEST"である
@@ -234,7 +176,7 @@ sub master_running{
     #      の結果が"PRI"である
     ##
 
-    $command = "$CRM_RESOURCE -r $master_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
+    $command = "$CRM_RESOURCE -r $primary_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
     if ($ssh_info){
         @results = ssh_exec_command($ssh_info, $command);
     }
@@ -242,7 +184,7 @@ sub master_running{
         $results[0] = `$command`;
         chomp $results[0];
     }
-    if ($results[0] ne "resource $master_resource_id is running on: $my_node Master"){
+    if ($results[0] ne "resource $primary_resource_id is running on: $my_node Master"){
         return 0;
     }
 
@@ -274,8 +216,8 @@ sub master_running{
 }
 
 
-sub slave_running{
-    my ($my_node, $master_resource_id, $pg_primitive_resource_id, $error_code) = @_;
+sub standby_running{
+    my ($my_node, $primary_resource_id, $pg_primitive_resource_id, $error_code) = @_;
     my $command;
     my $result;
     my $pgsql_data_status;
@@ -284,8 +226,8 @@ sub slave_running{
     ##
     # 以下の条件を満たしたとき結果を OK とする
     #   *  コマンド
-    #     「crm_resource -r <MasterリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
-    #      の結果が"resource <MasterリソースID> is running on: <マシンのホスト名> "である
+    #     「crm_resource -r <PrimaryリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
+    #      の結果が"resource <PrimaryリソースID> is running on: <マシンのホスト名> "である
     #   *  コマンド
     #     「crm_attribute -l forever -N <マシンのホスト名> --name <PostgreSQLのPrimitiveのリソースID>-data-status -G -q」
     #      の結果が"STREAMING|SYNC"である
@@ -294,11 +236,11 @@ sub slave_running{
     #      の結果が"HS:SYNC"である
     ##
 
-    $command = "$CRM_RESOURCE -r $master_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
+    $command = "$CRM_RESOURCE -r $primary_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
     $result = `$command`;
     chomp $result;
 
-    if ($result ne "resource $master_resource_id is running on: $my_node "){
+    if ($result ne "resource $primary_resource_id is running on: $my_node "){
         if ($error_code){
             $$error_code = 1;
         }
@@ -410,41 +352,6 @@ sub ping_running{
 
 }
 
-
-sub diskd_running{
-    my ($my_node, $resource_id) = @_;
-    my $result;
-    my @resource_id;
-    my $array_num;
-    my $resource_check_count = 0;
-
-    ##
-    # 以下の条件を指定されたリソースの数だけ満たしたとき結果を OK とする
-    #   *  コマンド
-    #     「crm_resource -r <リソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
-    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
-    ##
-
-    @resource_id = split(/\s*,\s*/, $resource_id);
-    $array_num = scalar(@resource_id);
-    
-    foreach my $id (@resource_id){
-        $result = `$CRM_RESOURCE -r $id -W 2> /dev/null | $GREP \" $my_node \"`;
-        chomp $result;
-        if ($result eq "resource $id is running on: $my_node "){
-            $resource_check_count ++;
-        }
-    }
-
-    if ($resource_check_count == $array_num){
-        return 1;
-    }
-    
-    return 0;
-
-}
-
-
 sub read_cib{
     my ($cib_path, $pg_primitive_resource_id, $kill_when_no_data) = @_;
     my @cib_strings;
@@ -473,10 +380,10 @@ sub read_cib{
         #     <nvpair name="<name>" value="<value>" id="<PostgreSQLのPrimitiveのリソースID>-instance_attributes-<文字列>" />
         #     ※neme, value, id タグの順番が変わっても対応できるようにパラメータ値を取得する
         if ($line =~ /.*id=\"$pg_primitive_resource_id-instance_attributes-\S+\"/){
-            if ($line =~ /\s+name=\"([^\"\s]+)\"\s+/){
+            if ($line =~ /\s+name=\"([^\"\s]+)\"/){
                 $param_name = $1;
             }
-            if ($line =~ /\s+value=\"([^\"]+)\"\s+/){
+            if ($line =~ /\s+value=\"([^\"]+)\"/){
                 $param_value = $1;
             }
             if ($param_name && $param_value) {
@@ -513,12 +420,12 @@ sub read_config{
     my ($config_path) = @_;
     my @config_strings;
     my %config_value;
-    my @check_key_list = ("Archive_dir", "D-LAN_IPAddress", 
-                         "PG-REX_Master_ResourceID","PG-REX_Primitive_ResourceID");
+    my @check_key_list = ("Archive_dir", "D_LAN_IPAddress", 
+                         "PG_REX_Primary_ResourceID","PG_REX_Primitive_ResourceID","IC_LAN_IPAddress","HACLUSTER_NAME");
     my $result;
 
     $config_value{'STONITH'} = "enable";
-    $config_value{'VIP_SLAVE'} = "enable";
+    $config_value{'IPADDR_STANDBY'} = "enable";
     $config_value{'PEER_NODE_SSH_PASS_MODE'} = "manual";
     $config_value{'BACKUP_NODE_SSH_PASS_MODE'} = "manual";
     
@@ -553,10 +460,10 @@ sub read_config{
     if ($config_value{'Archive_dir'} !~ /^\//){
         printlog("ERROR", COMMON_MS0005, "Archive_dir");
     }
-    if ($config_value{'STONITH'} ne "enable" && $config_value{'STONITH'} ne "disable"){
+    if ($config_value{'STONITH'} ne "enable"){
         printlog("ERROR", COMMON_MS0006);
     }
-    if ($config_value{'VIP_SLAVE'} ne "enable" && $config_value{'VIP_SLAVE'} ne "disable"){
+    if ($config_value{'IPADDR_STANDBY'} ne "enable" && $config_value{'IPADDR_STANDBY'} ne "disable"){
         printlog("ERROR", COMMON_MS0021);
     }
     if ($config_value{'PEER_NODE_SSH_PASS_MODE'} ne "manual" && $config_value{'PEER_NODE_SSH_PASS_MODE'} ne "passfile"
@@ -574,6 +481,11 @@ sub read_config{
         if ($config_value{'PEER_NODE_SSH_PASS_FILE'} !~ /^\//){
           printlog("ERROR", COMMON_MS0005, "PEER_NODE_SSH_PASS_FILE");
         }
+        my $mode = (stat($config_value{'PEER_NODE_SSH_PASS_FILE'} ))[2];
+        my $passfile_rwx = sprintf ("%o", S_IMODE($mode));
+        if ($passfile_rwx ne "600"){
+          printlog("ERROR", COMMON_MS0058, $config_value{'PEER_NODE_SSH_PASS_FILE'});
+        }
     }
     if ($config_value{'BACKUP_NODE_SSH_PASS_MODE'} eq "passfile"){
         if (!defined($config_value{'BACKUP_NODE_SSH_PASS_FILE'})){
@@ -582,10 +494,15 @@ sub read_config{
         if ($config_value{'BACKUP_NODE_SSH_PASS_FILE'} !~ /^\//){
           printlog("ERROR", COMMON_MS0005, "BACKUP_NODE_SSH_PASS_FILE");
         }
+        my $mode = (stat($config_value{'BACKUP_NODE_SSH_PASS_FILE'} ))[2];
+        my $passfile_rwx = sprintf ("%o", S_IMODE($mode));
+        if ($passfile_rwx ne "600"){
+          printlog("ERROR", COMMON_MS0058, $config_value{'BACKUP_NODE_SSH_PASS_FILE'});
+        }
     }
-    # VIP_SLAVE が disable の場合、VIP-SLAVE_ResourceID の値を空にする
-    if ($config_value{'VIP_SLAVE'} eq "disable"){
-        $config_value{'VIP-SLAVE_ResourceID'} = "";
+    # IPADDR_STANDBY が disable の場合、IPADDR_STANDBY_ResourceID の値を空にする
+    if ($config_value{'IPADDR_STANDBY'} eq "disable"){
+        $config_value{'IPADDR_STANDBY_ResourceID'} = "";
     }
 
     # STONITH が enable の場合、 Group のリソース ID を配列に分解しハッシュに格納する
@@ -603,9 +520,41 @@ sub read_config{
 
     my @ifconfig_ip = ();
     # ","で区切られた D-LAN の IP アドレスをそれぞれ配列に格納
-    my @dlan_ipaddr = split(/\s*,\s*/, $config_value{'D-LAN_IPAddress'});
+    my @dlan_ipaddr = split(/\s*,\s*/, $config_value{'D_LAN_IPAddress'});
     if (scalar(@dlan_ipaddr) != 2){
-        printlog("ERROR", COMMON_MS0007);
+        printlog("ERROR", COMMON_MS0007, "D_LAN_IPAddress");
+    }
+
+    # IC_LAN_IPAddressのパース
+    my $iclan_ipaddr1;
+    my $iclan_ipaddr2;
+    # IC_LAN_IPAddressのIC-LANが1つで、下記の形式の場合
+    # (IPアドレス)
+    if ( $config_value{'IC_LAN_IPAddress'} =~ /^\s*\(\s*([^\)]+)\s*\)\s*$/){
+        $iclan_ipaddr1 = $1;
+        $iclan_ipaddr1  =~ s/^ *(.*?) *$/$1/;
+    # IC_LAN_IPAddressのIC-LANが2つで、下記の形式の場合
+    # (IPアドレス,IPアドレス)
+    } elsif ( $config_value{'IC_LAN_IPAddress'} =~ /^\s*\(\s*([^\)]+)\s*\)\s*,\s*\(\s*([^\)]+)\s*\)\s*$/ ) {
+        $iclan_ipaddr1 = $1;
+        $iclan_ipaddr2 = $2;
+        $iclan_ipaddr1  =~ s/^ *(.*?) *$/$1/;
+        $iclan_ipaddr2  =~ s/^ *(.*?) *$/$1/;
+    } else {
+        printlog("ERROR", COMMON_MS0057, "IC_LAN_IPAddress");
+    }
+
+    # ","で区切られた IC-LAN の IP アドレスをそれぞれ配列に格納
+    my @iclan_ipaddr1 = split(/\s*,\s*/, $iclan_ipaddr1);
+    if (scalar(@iclan_ipaddr1) != 2){
+        printlog("ERROR", COMMON_MS0007, "IC_LAN_IPAddress");
+    }
+    my @iclan_ipaddr2;
+    if(defined($iclan_ipaddr2)){
+        @iclan_ipaddr2 = split(/\s*,\s*/, $iclan_ipaddr2);
+        if (scalar(@iclan_ipaddr2) != 2){
+            printlog("ERROR", COMMON_MS0007, "IC_LAN_IPAddress");
+        }
     }
 
     # コマンド実行マシンに設定してある IP アドレスを配列に格納
@@ -624,179 +573,61 @@ sub read_config{
 
     foreach my $line (@dlan_ipaddr){
         if (grep {$_ eq $line} @ifconfig_ip){
-            $config_value{'My_D-LAN_IPAddress'} = $line;
+            $config_value{'My_D_LAN_IPAddress'} = $line;
         }
         else {
-            $config_value{'Another_D-LAN_IPAddress'} = $line;
+            $config_value{'Another_D_LAN_IPAddress'} = $line;
         }
     }
 
-    if (!exists($config_value{'My_D-LAN_IPAddress'})){
-        printlog("ERROR", COMMON_MS0009);
+    if (!exists($config_value{'My_D_LAN_IPAddress'})){
+        printlog("ERROR", COMMON_MS0009, "D_LAN_IPAddress", $config_value{'D_LAN_IPAddress'});
     }
-    if (!exists($config_value{'Another_D-LAN_IPAddress'})){
-        printlog("ERROR", COMMON_MS0010);
+    if (!exists($config_value{'Another_D_LAN_IPAddress'})){
+        printlog("ERROR", COMMON_MS0010, "D_LAN_IPAddress", $config_value{'D_LAN_IPAddress'});
+    }
+
+    foreach my $line (@iclan_ipaddr1){
+        if (grep {$_ eq $line} @ifconfig_ip){
+            $config_value{'My_IC_LAN_IPAddress1'} = $line;
+        }
+        else {
+            $config_value{'Another_IC_LAN_IPAddress1'} = $line;
+        }
+    }
+
+    if (!exists($config_value{'My_IC_LAN_IPAddress1'})){
+        printlog("ERROR", COMMON_MS0009, "IC_LAN_IPAddress", "($iclan_ipaddr1)");
+    }
+    if (!exists($config_value{'Another_IC_LAN_IPAddress1'})){
+        printlog("ERROR", COMMON_MS0010, "IC_LAN_IPAddress", "($iclan_ipaddr1)");
+    }
+
+    if (defined($iclan_ipaddr2)){
+        foreach my $line (@iclan_ipaddr2){
+            if (grep {$_ eq $line} @ifconfig_ip){
+                $config_value{'My_IC_LAN_IPAddress2'} = $line;
+            }
+            else {
+                $config_value{'Another_IC_LAN_IPAddress2'} = $line;
+            }
+        }
+
+        if (!exists($config_value{'My_IC_LAN_IPAddress2'})){
+            printlog("ERROR", COMMON_MS0009, "IC_LAN_IPAddress", "($iclan_ipaddr2)");
+        }
+        if (!exists($config_value{'Another_IC_LAN_IPAddress2'})){
+            printlog("ERROR", COMMON_MS0010, "IC_LAN_IPAddress", "($iclan_ipaddr2)");
+        }
+    }
+    if ( $config_value{'HACLUSTER_NAME'} !~ /^[_0-9A-Za-z]+[\-_0-9A-Za-z]*$/){
+        printlog("ERROR", COMMON_MS0057, "HACLUSTER_NAME");
     }
 
     return %config_value;
 }
 
 
-sub read_hacf{
-    my ($hacf_path, $status) = @_;
-    my @hacf_strings;
-    my %hacf_value;
-    my @iclan_netaddr;
-
-    open (FILE, $hacf_path) or printlog("ERROR", COMMON_MS0025, $hacf_path);
-    @hacf_strings = <FILE>;
-    close (FILE);
-
-    # IC-LAN に使用しているネットワークアドレス (または IP アドレス) を取得する
-    foreach my $line (@hacf_strings){
-        chomp $line;
-        # comment format : #<comment>
-        $line =~ s/\#.*//g;
-        # format of bindnetaddr in corosync.conf: bindnetaddr: <network address> or <IP adress>
-        if ($line =~ /^\s*bindnetaddr:\s+(\S+)(\s.*)?$/){
-            push(@iclan_netaddr, $1);
-        }
-    }
-
-    # 起動処理の時のみIC-LAN に使用しているネットワークアドレス (または IP アドレス)
-    # の数を確認する
-    if ($status && scalar(@iclan_netaddr) == 0){
-        printlog("ERROR", COMMON_MS0019);
-    }
-    if ($status && scalar(@iclan_netaddr) == 1){
-        printlog("LOG", COMMON_MS0027);
-    }
-
-    %hacf_value = (iclan_netaddr => [@iclan_netaddr]);
-
-    return %hacf_value;
-}
-
-
-sub get_another_iclan_netaddr{
-    my ($hacf_path, $ssh_info) = @_;
-    my @hacf_strings;
-    my %hacf_value;
-    my @iclan_netaddr;
-    my $command;
-    my @results;
-
-    $command = "$CAT $hacf_path";
-    @results = ssh_exec_command($ssh_info, $command);
-    if (!$results[0]){
-        printlog("ERROR", COMMON_MS0041, $hacf_path);
-    }
-    @hacf_strings = split (/\n/, $results[0]);
-    # IC-LAN に使用しているネットワークアドレス (または IP アドレス) を取得する
-    # Master または Slave 起動時に bindnetaddr が 2 つ設定されていることは確認済み
-    # であるため、ここではチェックしない
-    foreach my $line (@hacf_strings){
-        chomp $line;
-        # comment format : #<comment>
-        $line =~ s/\#.*//g;
-        # IC-LAN network address format : bindnetaddr: <network address> or <IP adress>
-        if ($line =~ /^\s*bindnetaddr:\s+(\S+)/){
-            push(@iclan_netaddr, $1);
-        }
-    }
-    return @iclan_netaddr;
-}
-
-
-sub get_iclan_ipaddr{
-    my ($my_iclan_netaddr, $another_iclan_netaddr, $node_value, $ssh_info) = @_;
-    my @iclan_netaddr = ($my_iclan_netaddr, $another_iclan_netaddr);
-    my @node_list = ($$node_value{'my_node'}, $$node_value{'another_node'});
-    my @site = ("my_node","another_node");
-    my $netaddr;
-    my $result = "";
-    my @results;
-    my @field;
-    my %ipaddr_list;
-    my $nic;
-    my %iclan_ipaddr;
-    my $command;
-    my @record;
-
-    # 自身のノードと相手先のノードの IC-LAN IP アドレスを取得する
-    for (my $count = 0; $count < 2; $count++){
-        foreach $netaddr (@{$iclan_netaddr[$count]}){
-            # bindnetaddr に指定されたアドレスが NIC に設定されている場合は、
-            # bindnetaddr にホストアドレスが設定されていると判断し、ifconfig
-            # コマンドで確認した上で bindnetaddrの設定値を IC-LAN IP アドレス
-            # として取得する
-            $command = "$IFCONFIG | $GREP -F \" $netaddr \"";
-            if ($count == 0){
-                $results[0] = `$command`;
-            }
-            else {
-                @results = ssh_exec_command($ssh_info, $command);
-            }
-            if ($results[0] =~ /inet\s(addr:)?([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)\s/) {
-                if ($2 eq $netaddr){
-                    if (!is_ipv4_format($2)){
-                        printlog("ERROR", COMMON_MS0052, $2);
-                    }
-                    push(@{$ipaddr_list{$site[$count]}}, $2);
-                    next;
-                }
-            }
-
-            # bindnetaddr にネットワークアドレスが設定されている場合は、当該アドレスが
-            # 設定されている NIC を特定する
-            $command = "$ROUTE";
-            if ($count == 0){
-                $results[0] = `$command`;
-            }
-            else {
-                @results = ssh_exec_command($ssh_info, $command);
-            }
-
-            if (!$results[0]){
-                printlog("ERROR", COMMON_MS0039, $node_list[$count], $netaddr);
-            }
-
-            @record = split (/\n/, $results[0]);
-            foreach (@record) {
-                # routeのコマンドの実行結果はスペース区切りとなるため、
-                # スペースをデリミタとしてsplitする
-                # $element[0]にIPアドレス、$element[7]にNIC名が格納される
-                my @element = split(/\s+/, $_);
-                if ($element[0] eq $netaddr) {
-                        $nic = $element[7];
-                }
-            }
-
-            # 特定した NIC に設定されている IC-LAN IP アドレスを取得する
-            $command = "$IFCONFIG \"$nic\"";
-            if ($count == 0){
-                $results[0] = `$command`;
-            }
-            else {
-                @results = ssh_exec_command($ssh_info, $command);
-            }
-
-            if ($results[0] =~ /inet\s(addr:)?(\S+)\s/){
-                if (!is_ipv4_format($2)){
-                    printlog("ERROR", COMMON_MS0052, $2);
-                }
-                push(@{$ipaddr_list{$site[$count]}}, $2);
-            }
-            else {
-                printlog("ERROR", COMMON_MS0040, $node_list[$count], $netaddr);
-            }
-        }
-        @{$iclan_ipaddr{$site[$count]}} = @{$ipaddr_list{$site[$count]}};
-    }
-    return %iclan_ipaddr;
-}
-
-
 sub exec_command {
     my ($command) = @_;
     my $result;
@@ -1071,13 +902,13 @@ sub check_support_version {
 
     # Pacemaker がサポート対象バージョンであるかを確認する
     @version_factor = split(/\./, get_pm_version());
-    if (!($version_factor[0] == 1 && $version_factor[1] == 1)){
+    if (!($version_factor[0] == 2 && $version_factor[1] == 0)){
         printlog("ERROR", COMMON_MS0038, get_pm_version());
     }
 
     # PostgreSQL がサポート対象バージョンであるかを確認する
     $version_num = get_pg_version_num($postgres_path, $exec_user);
-    if (int($version_num / 10000) != 11){
+    if (int($version_num / 10000) != 12){
         if ($version_num >= 100000) {
             printlog("ERROR", COMMON_MS0037, int($version_num / 10000));
         } else {
@@ -1087,24 +918,6 @@ sub check_support_version {
     }
 }
 
-sub ifcheckd_running {
-    my ($ssh_info) = @_;
-    my $command = "$PIDOF ifcheckd > /dev/null";
-    my $exit_code;
-    my @results;
-
-    `$command`;
-     $exit_code = $? >> 8;
-     if ($exit_code != 0) {
-         printlog("ERROR", COMMON_MS0042);
-     }
-
-     @results = ssh_exec_command($ssh_info, $command);
-     if ($results[1] != 0) {
-         printlog("ERROR", COMMON_MS0043);
-     }
-}
-
 sub get_pg_command_path {
     my ($pg_path, $exec_user) = @_;
     my @command_list = ("postgres", "psql", "pg_controldata", "pg_basebackup", "pg_waldump", "pg_rewind", "pg_ctl");
@@ -1159,7 +972,7 @@ sub get_pm_version{
     my $result;
     my @results;
 
-    $result = exec_command("$CRM_MON --version");
+    $result = exec_command("$PACEMAKERD --version");
 
     @results = split(/\s/,$result);
     chomp $results[1];
@@ -1289,14 +1102,16 @@ sub check_dbcluster_access {
     open my $in, '-|', "$LSOF +D $cluster_path -F cuLR0 2> /dev/null" || printlog("ERROR", COMMON_MS0046, $!);
 	# Suppress display of stderr only, because error of lsof command is unknown
 
+    my $pid = 0;
     while (<$in>) {
-        my $pid;
-
         while (/([a-zA-Z])([^\x00]+)\x00/g) {
             if ($1 eq "p") {
                 $pid = $2;
+            } elsif ($pid == 0 || ($1 ne 'f' && defined $procs{$pid}{$1})) {
+                printlog("ERROR", COMMON_MS0056, ": $_\n");
             } else {
-                $procs{$pid}{$1} = $2;
+                $procs{$pid}{$1} .= ', ' if (defined $procs{$pid}{$1});
+                $procs{$pid}{$1} .= $2;
             }
         }
     }
